package net.duiker101.opendungeon;

import java.util.ArrayList;
import java.util.HashMap;

import org.anddev.andengine.engine.Engine;
import org.anddev.andengine.engine.handler.IUpdateHandler;
import org.anddev.andengine.entity.scene.Scene;
import org.anddev.andengine.entity.sprite.Sprite;
import org.anddev.andengine.input.touch.TouchEvent;

import android.graphics.Point;

public class BricksField {
	
	class CheckBlock{
		int checkA=0;//0 == not check   1=no match   2==matched
		int checkB=0;
		Point a=null;
		Point b=null;
	}
	
	private Game mGame;
	private Scene mScene;
	private Engine mEngine;
	
	private BrickTextures mTextures;
	
	private BrickControls controls;
	
	public HashMap<String,Brick> mBricks;
	public HashMap<String,String> mBricksMap;
	private ArrayList<CheckBlock> mToCheck;
	
	private Sprite mCursor;
	
	public int mPosX,mPosY,mWidth,mHeight,mXBricks,mYBricks;
	
	public float mBrickWidth,mBrickHeight;
	
	public BricksField(Game pGame){
		this.mGame=pGame;
		
		this.mPosX=0;
		this.mPosY=300;
		this.mWidth=480;
		this.mHeight=480;
		
		this.mXBricks=8;
		this.mYBricks=8;
		
		this.mBrickWidth=this.mWidth/mXBricks;
		this.mBrickHeight=this.mHeight/this.mYBricks;
		
		this.controls= new BrickControls(this);
		this.mTextures = new BrickTextures(this.mGame);
	}
	
	public void loadResource(){
		this.mTextures.loadResources();
	}
	
	public void loadScene(){
		this.mEngine=this.mGame.getEngine();
		this.mScene=this.mEngine.getScene();
		
		this.mScene.registerUpdateHandler(new IUpdateHandler(){
			@Override
			public void onUpdate(float pSecondsElapsed) {
				update();
			}
			@Override
			public void reset() {}
		});

		mCursor= new Sprite(0, 0, mTextures.cursorRegion);
		mCursor.setSize(mBrickWidth, mBrickHeight);
		mCursor.setZIndex(0);
	}

	public void generate(){
		
		mBricks=new HashMap<String, Brick>();
		mBricksMap=new HashMap<String, String>();
		mToCheck=new ArrayList<BricksField.CheckBlock>();
		for(int x=0;x<mXBricks;x++){
			
			for(int y=-mYBricks;y<mYBricks;y++){
				
				if(y<0){
					Brick b=new Brick(mGame,this,
							this.mTextures.bricksRegion,
							pToNum(x,y),
							x,y,
							mBrickWidth,mBrickHeight,
							mPosX,mPosY);
					
					mBricks.put(String.valueOf(pToNum(x,y+mYBricks)), b);
					mBricksMap.put(pToSCoord(x, y), pToSNum(x, y+mYBricks));
				}else{
					mBricksMap.put(pToSCoord(x, y),"x");
				}
			}
			
		}

		mScene.attachChild(mCursor);
		hideCursor();
	}

	
	//----------------------------------------------------------------------------------------------------------

	private void update(){
		checkFreeBlocks();
		checkChanges();
		
		ArrayList<Point> toDel=new ArrayList<Point>();
		calculate(toDel,true);
		calculate(toDel,false);

		for(int j=0;j<mToCheck.size();j++){
			if(mToCheck.get(j).checkA > 0 && mToCheck.get(j).checkB > 0){
				if(!toDel.contains(mToCheck.get(j).a) && !toDel.contains(mToCheck.get(j).b)){
					requestMove(mToCheck.get(j).a, mToCheck.get(j).b, false);
				}
				mToCheck.set(j, null);
			}
		}
		
		for(int j=mToCheck.size()-1;j>=0;j--){
			if(mToCheck.get(j)==null){
				mToCheck.remove(j);
			}
		}
		
		for(int i=0;i<toDel.size();i++){
			destroy(toDel.get(i).x, toDel.get(i).y);
		}
	}
	
	private void checkFreeBlocks(){
		for(int x=0;x<mXBricks;x++){
			boolean still=true;
			for(int y=-mYBricks;y<mYBricks;y++){
				String key=mBricksMap.get(pToSCoord(x, y));
				if(!key.equals("x") && !key.equals("o")){
					Brick tmp=mBricks.get(key);
					if(tmp.mMoving==true || tmp.mReqMove==true){
						still=false;
						break;
					}
				}
			}
			
			if(still==false)
				continue;
			
			boolean toMove=false;
			for(int y=mYBricks-1;y>=-mYBricks;y--){
				String key=mBricksMap.get(pToSCoord(x, y));
				if(toMove==true){
					mBricksMap.put(pToSCoord(x, y+1), key);
				}
				if( key.equals("x") && toMove == false){
					toMove=true;
				}
			}
			if(toMove==true)
				mBricksMap.put(pToSCoord(x, -mYBricks), "x");
			
		}
	}
	
	private void checkChanges(){
		for(int x=0;x<mXBricks;x++){									//CHECKS FOR MOVES
			for(int y=-mYBricks;y<mYBricks;y++){
				String key=mBricksMap.get(pToSCoord(x, y));
				if(!key.equals("x") && !key.equals("o")){
					Brick tmp=mBricks.get(key);
					if(tmp.mMoving==false){
						if(tmp.getPos().x != x || tmp.getPos().y != y)
							tmp.moveTo(x, y);
					}
				}
			}
		}
	}
	
	private void calculate(ArrayList<Point> list, boolean isX){
		for(int x=0;x<mXBricks;x++){
			int cType=-1;
			int cLastType=-1;
			ArrayList<Point> totfound=new ArrayList<Point>();
			ArrayList<Point> found=new ArrayList<Point>();
			
			for(int y=0;y<mYBricks;y++){
				String key="";
				Point point=null;
				if(isX==true){
					key=mBricksMap.get(pToSCoord(x, y));
					point=new Point(x, y);
				}else{
					key=mBricksMap.get(pToSCoord(y, x));
					point=new Point(y, x);
				}
				boolean toClear=false,toAdd=false;
				
				if(!key.equals("x") && !key.equals("o")){
					Brick tmp=mBricks.get(key);
					cType=tmp.mType;
					if(tmp.mMoving==false && tmp.mReqMove==false){
						for(int jj=0;jj<mToCheck.size();jj++){
							if(point.x == mToCheck.get(jj).a.x && point.y == mToCheck.get(jj).a.y){
								mToCheck.get(jj).checkA=1;
							}
							if(point.x == mToCheck.get(jj).b.x && point.y == mToCheck.get(jj).b.y){
								mToCheck.get(jj).checkB=1;
							}
						}
						toAdd=true;
						if(cType!=cLastType)
							toClear=true;
					}else
						toClear=true;
				}else
					toClear=true;
				
				if(toClear==true){
					if(found.size() >= 3)
						totfound.addAll(found);
					found.clear();
				}
				if(toAdd==true)
					found.add(point);
				cLastType=cType;	
				
			}

			if(found.size() >= 3)
				totfound.addAll(found);
			list.addAll(totfound);
		}
	}
	
	
	//----------------------------------------------------------------------------------------------------------	
	
	
	public int pToNum(int x, int y){
		return x*mYBricks+y;
	}
	public String pToSNum(int x, int y){
		return String.valueOf(x*mYBricks+y);
	}
	public String pToSCoord(int x, int y){
		return String.valueOf(x) + "-" + String.valueOf(y);
	}
	
	
	public void requestMove(Point pFrom, Point pTo,boolean	check){
		String f=pToSCoord(pFrom.x, pFrom.y);
		String t=pToSCoord(pTo.x, pTo.y);
		
		String fk=mBricksMap.get(f);
		String tk=mBricksMap.get(t);
		
		if(mBricks.get(fk).mMoving==true || mBricks.get(fk).mMoving==true)
			return;
		
		mBricksMap.put(f, tk);
		mBricksMap.put(t, fk);
		
		mBricks.get(fk).mReqMove=true;
		mBricks.get(tk).mReqMove=true;
		
		if(!check)
			return;
		
		CheckBlock cb= new CheckBlock();
		cb.a=pFrom;
		cb.b=pTo;
		mToCheck.add(cb);
	}
		
	public void destroy(int x,int y){
		int f=-1;
		for(int oy=-1;oy>-mYBricks;oy--){
			if(mBricksMap.get(pToSCoord(x, oy)).equals("x")){
				f=oy;
				break;
			}
		}
		Brick tmp = mBricks.get(mBricksMap.get(pToSCoord(x, y)));
		log("destory");
		tmp.destroy(x, y, x, f);
		mBricksMap.put(pToSCoord(x, f), "o");
	}
	
	public void setAfterDestory(int fX,int fY,int tX,int tY){
		String thisVal=mBricksMap.get(pToSCoord(fX, fY));
		mBricksMap.put(pToSCoord(tX, tY), thisVal);
		mBricksMap.put(pToSCoord(fX, fY), "x");
	}

	public boolean touch(TouchEvent pEvent,int pId,int x, int y) {
		return controls.touch(pEvent, pId, x,  y);
	}
	
	public void setCursor(int pX,int pY){
		if(!this.mCursor.isVisible())
			this.mCursor.setVisible(true);
		this.mCursor.setPosition(this.mPosX+(pX*this.mBrickWidth), this.mPosY+(pY*this.mBrickHeight));
	}
	
	public void hideCursor(){
		this.mCursor.setVisible(false);
	}
	
	public void log(Object obj){
		System.out.println(obj);
	}
	
}
